#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "system.h"
#include "ran_uniform.h"

void Mcloop(void)
{
  int i,j,k;
  int NumberAccepted,NumberTrials;
  double RosenbluthFactor,RosenbluthFactorOld,r;
  double BondedEnergyOld,BondedEnergyNew,NonBondedEnergyNew,NonBondedEnergyOld;
  double BondedEnergySum,NonBondedEnergySum,EnergyNormalize,BondedEnergy,NonBondedEnergy;
  FILE *FilePtr;

  FilePtr=fopen("movie.pdb","w");
  InitializePdb(FilePtr);

  // generate a first chain
  RosenbluthFactor=1.0;
  BondedEnergyNew=0.0;
  NonBondedEnergyNew=0.0;
  RosenbluthFactorOld=1.0;
  BondedEnergyOld=0.0;
  NonBondedEnergyOld=0.0;
  NumberTrials=0.0;
  NumberAccepted=0.0;
  BondedEnergySum=0.0;
  NonBondedEnergySum=0.0;
  EnergyNormalize=0.0;
      
  Grow(FALSE,&RosenbluthFactorOld,&BondedEnergyOld,&NonBondedEnergyOld);


  for(j=0;j<ChainLength;j++)
  {
    Positions[j].x=TrialPositions[j].x;
    Positions[j].y=TrialPositions[j].y;
    Positions[j].z=TrialPositions[j].z;
  }

  BondedEnergy=BondedEnergyOld;
  NonBondedEnergy=NonBondedEnergyOld;

  Sample(INITIALIZE,1.0,1.0);

  for(i=0;i<NumberOfSteps;i++)
  {
    for(k=0;k<1000;k++)
    {
      // static scheme
      if(Lstatic)
      {
        Grow(FALSE,&RosenbluthFactor,&BondedEnergyNew,&NonBondedEnergyNew);

        for(j=0;j<ChainLength;j++)
        {
          Positions[j].x=TrialPositions[j].x;
          Positions[j].y=TrialPositions[j].y;
          Positions[j].z=TrialPositions[j].z;
        }

        BondedEnergySum+=BondedEnergyNew*RosenbluthFactor;
        NonBondedEnergySum+=NonBondedEnergyNew*RosenbluthFactor;
        EnergyNormalize+=RosenbluthFactor;
      }
      else
      {
        // dynamic scheme
        NumberTrials++;
        Grow(TRUE,&RosenbluthFactorOld,&BondedEnergyOld,&NonBondedEnergyOld);
        Grow(FALSE,&RosenbluthFactor,&BondedEnergyNew,&NonBondedEnergyNew);

        // accepted ? then copy coordinates
        if(RandomNumber()<(RosenbluthFactor/RosenbluthFactorOld))
        {
          NumberAccepted++;
          NonBondedEnergy=NonBondedEnergyNew;
          BondedEnergy=BondedEnergyNew;

          for(j=0;j<ChainLength;j++)
          {
            Positions[j].x=TrialPositions[j].x;
            Positions[j].y=TrialPositions[j].y;
            Positions[j].z=TrialPositions[j].z;
          }
        }
        RosenbluthFactor=1.0;
      }

      // sample end-to-end distance and average energies
      r=sqrt(SQR(Positions[ChainLength-1].x-Positions[0].x)+
              SQR(Positions[ChainLength-1].y-Positions[0].y)+
              SQR(Positions[ChainLength-1].z-Positions[0].z));
      if(i>=NumberOfInitializationSteps) 
      {
        Sample(SAMPLE,r,RosenbluthFactor);

        if(!Lstatic)
        {
          BondedEnergySum+=BondedEnergy;
          NonBondedEnergySum+=NonBondedEnergy;
          EnergyNormalize+=1.0;
        }
        if(k==0&&(i%5)==0) 
          WritePdb(FilePtr);
      }
    }
  }

  Sample(WRITE_RESULTS,1.0,1.0);

  if(!Lstatic) printf("Fraction accepted : %f\n",(double)NumberAccepted/NumberTrials);

  printf("Average bonded energy       : %f\n",BondedEnergySum/EnergyNormalize);
  printf("Average nonbonded energy    : %f\n",NonBondedEnergySum/EnergyNormalize);

  fclose(FilePtr);
}
